package com.erinors.tapestry.tapdoc.maven;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;

public class CommandlineJavadocExecutor implements JavadocExecutor
{
    private boolean trimClasspath;
    
    public CommandlineJavadocExecutor(boolean trim) {    
        trimClasspath = trim;
    }
        
    public void runJavadoc(Collection<CharSequence> classpaths, CharSequence docletpath, CharSequence doclet,
            CharSequence outputDirectory, Collection<CharSequence> sourcepaths, CharSequence packages)
    {
        StringBuilder formattedClasspath = new StringBuilder();
        for (CharSequence file : classpaths)
        {
            if (!valid(file))
                continue;
            
            if (formattedClasspath.length() > 0)
            {                
                formattedClasspath.append(File.pathSeparatorChar);
            }
            
            formattedClasspath.append('\"');
            formattedClasspath.append(file);
            formattedClasspath.append('\"');
        }
        
        StringBuilder formattedSourcePath = new StringBuilder();
        for (CharSequence file : sourcepaths)
        {
            if (formattedSourcePath.length() > 0)
            {
                formattedSourcePath.append(File.pathSeparatorChar);
            }
            
            formattedSourcePath.append('\"');
            formattedSourcePath.append(file);
            formattedSourcePath.append('\"');
        }
        
        StringBuilder command = new StringBuilder();
        command.append("java ");
        command.append("-cp ");
        command.append(formattedClasspath);
        command.append(" com.sun.tools.javadoc.Main ");
        command.append("-docletpath \"");
        command.append(docletpath);
        command.append("\" -doclet ");
        command.append(doclet);
        command.append(" -d \"");
        command.append(outputDirectory);
        command.append("\" -sourcepath \"");
        command.append(formattedSourcePath);
        command.append("\" -subpackages ");
        command.append(packages);

        System.out.println(command);
        
        String osName = System.getProperty("os.name");
        String[] commands = null;

        if (osName.startsWith("Windows"))
        {
            commands = new String[]{"cmd.exe", "/c", command.toString()};
        } 
        else
        {
            commands = new String[]{"sh", "-c", command.toString()};
        }        
        
        try
        {
            Process process = Runtime.getRuntime().exec(commands);
            //process.waitFor();
            output("Error", process.getErrorStream());
            output("Input", process.getInputStream());            
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        /*catch (InterruptedException  e)
        {
            e.printStackTrace();
        } */       
    }

    private void output(String label, InputStream stream) {
        try {
            if (stream.available()<=0)
                return;
            System.out.println(label + " : ");
            DataInputStream dis = new DataInputStream(stream);
            System.out.println(dis.readLine());
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    private boolean valid(CharSequence file) {
        String name = file.toString();
        return !trimClasspath || name.contains("tapestry") || name.contains("tapdoc") ||
                name.contains("hivemind") || name.contains("tools");
    }
}
