/*
 *  Copyright 2007 andyhot.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */

package com.erinors.tapestry.tapdoc.maven;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Properties;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.codehaus.plexus.util.cli.CommandLineUtils;

/**
 *
 * @author andyhot
 */
public class DirectJavadocExecutor implements JavadocExecutor {
    
    private String javadocExecutable;

    public void runJavadoc(Collection<CharSequence> classpaths, CharSequence docletpath, CharSequence doclet, 
            CharSequence outputDirectory, Collection<CharSequence> sourcepaths, CharSequence packages) {

StringBuilder formattedClasspath = new StringBuilder();
        for (CharSequence file : classpaths)
        {
            if (formattedClasspath.length() > 0)
            {                
                formattedClasspath.append(File.pathSeparatorChar);
            }
            
            formattedClasspath.append('\"');
            formattedClasspath.append(file);
            formattedClasspath.append('\"');
        }
        
        StringBuilder formattedSourcePath = new StringBuilder();
        for (CharSequence file : sourcepaths)
        {
            if (formattedSourcePath.length() > 0)
            {
                formattedSourcePath.append(File.pathSeparatorChar);
            }
            
            formattedSourcePath.append('\"');
            formattedSourcePath.append(file);
            formattedSourcePath.append('\"');
        }
        
        StringBuilder command = new StringBuilder();        
        try {
            command.append(getJavadocExecutable());
        } catch (IOException e) {
            command.append("java ");
            //command.append(" com.sun.tools.javadoc.Main ");                        
        }
        command.append(" -cp ");
        command.append(formattedClasspath);
        command.append(" -docletpath \"");
        command.append(docletpath);
        command.append("\" -doclet ");
        command.append(doclet);
        command.append(" -d \"");
        command.append(outputDirectory);
        command.append("\" -sourcepath \"");
        command.append(formattedSourcePath);
        command.append("\" -subpackages ");
        command.append(packages);

        System.out.println(command);
        
        try
        {
            Runtime.getRuntime().exec(command.toString());
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }        
        
    }
    
    private String getJavadocExecutable()
        throws IOException
    {
        String javadocCommand = "javadoc" + ( SystemUtils.IS_OS_WINDOWS ? ".exe" : "" );

        File javadocExe;

        // ----------------------------------------------------------------------
        // The javadoc executable is defined by the user
        // ----------------------------------------------------------------------
        if ( StringUtils.isNotEmpty( javadocExecutable ) )
        {
            javadocExe = new File( javadocExecutable );

            if ( !javadocExe.exists() || !javadocExe.isFile() )
            {
                throw new IOException( "The javadoc executable '" + javadocExe + "' doesn't exist or is not a file. "
                    + "Verify the <javadocExecutable/> parameter." );
            }

            return javadocExe.getAbsolutePath();
        }

        // ----------------------------------------------------------------------
        // Try to find javadocExe from System.getProperty( "java.home" )
        // By default, System.getProperty( "java.home" ) = JRE_HOME and JRE_HOME
        // should be in the JDK_HOME
        // ----------------------------------------------------------------------
        // For IBM's JDK 1.2
        if ( SystemUtils.IS_OS_AIX )
        {
            javadocExe = new File( SystemUtils.getJavaHome() + File.separator + ".." + File.separator + "sh",
                                   javadocCommand );
        }
        else if ( SystemUtils.IS_OS_MAC_OSX )
        {
            javadocExe = new File( SystemUtils.getJavaHome() + File.separator + "bin", javadocCommand );
        }
        else
        {
            javadocExe = new File( SystemUtils.getJavaHome() + File.separator + ".." + File.separator + "bin",
                                   javadocCommand );
        }

        // ----------------------------------------------------------------------
        // Try to find javadocExe from JAVA_HOME environment variable
        // ----------------------------------------------------------------------
        if ( !javadocExe.exists() || !javadocExe.isFile() )
        {
            Properties env = CommandLineUtils.getSystemEnvVars();
            String javaHome = env.getProperty( "JAVA_HOME" );
            if ( StringUtils.isEmpty( javaHome ) )
            {
                throw new IOException( "The environment variable JAVA_HOME is not correctly set." );
            }
            if ( ( !new File( javaHome ).exists() ) || ( !new File( javaHome ).isDirectory() ) )
            {
                throw new IOException( "The environment variable JAVA_HOME=" + javaHome + " doesn't exist or is "
                    + "not a valid directory." );
            }

            javadocExe = new File( env.getProperty( "JAVA_HOME" ) + File.separator + "bin", javadocCommand );
        }

        if ( !javadocExe.exists() || !javadocExe.isFile() )
        {
            throw new IOException( "The javadoc executable '" + javadocExe + "' doesn't exist or is not a file. "
                + "Verify the JAVA_HOME environment variable." );
        }

        return javadocExe.getAbsolutePath();
    }    

}
